from core_tools.data.SQL.connect import set_up_remote_storage
from core_tools.data.gui.qml.data_browser import data_browser
from core_tools.data.ds.data_set import load_by_uuid
from core_tools.data.ds.ds2xarray import ds2xarray
import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
from cycler import cycler
from qcodes.utils import helpers
from scipy.optimize import curve_fit
import json
from scipy.constants import hbar, k, e, m_e, h
from scipy.signal import find_peaks
import xarray as xr
from core_tools.data.ds.ds_hdf5 import load_hdf5_uuid
import matplotlib.gridspec as gridspec

def get_param_val(ds, param):
    return json.loads(ds.attrs['snapshot'])['station']['parameters'][param]['value']

def linear(x, a, b):
    return a*x+b

def getDensity(rho_xy, field_analog, maxfield):
    try:
        coeff = abs(rho_xy).where(abs(field_analog)<maxfield, drop=True).curvefit(coords='field_analog', func=linear).curvefit_coefficients
        n = 1/(coeff[0]*e) / (1e4)
    except:
        popt, pcov = curve_fit(linear, field_analog, rho_xy)
        n = 1/(popt[0]*e) / (1e4)
    return n

def symmetrize(uuids, aspect_ratio, prominence, top_peaks=True, plot_figures=True, field_shift=None):      
    dsx1 = ds2xarray(load_hdf5_uuid(uuids[0], path)).swap_dims({'reps': 'field_analog'}).dropna('field_analog').sortby('field_analog')
    dsx2 = ds2xarray(load_hdf5_uuid(uuids[1], path)).swap_dims({'reps': 'field_analog'}).dropna('field_analog').sortby('field_analog')
 
    dsx1 = dsx1.sortby('field_analog')    
    dsx2 = dsx2.sortby('field_analog') 
    
    ## Match analog field to measured field
    if dsx1.field.mean() < 0:
        dsx1['field'] = -dsx1.field
    if dsx2.field.mean() < 0:
        dsx2['field'] = -dsx2.field
    
    ## Correct analog field to measured field
    if dsx1.field.mean() < dsx1.field_analog.mean():
        dsx1['field_analog'] = dsx1.field_analog - abs(dsx1.field.mean() - dsx1.field_analog.mean())
    else:
        dsx1['field_analog'] = dsx1.field_analog + abs(dsx1.field.mean() - dsx1.field_analog.mean())   
    if dsx2.field.mean() < dsx2.field_analog.mean():
        dsx2['field_analog'] = dsx2.field_analog - abs(dsx2.field.mean() - dsx2.field_analog.mean())
    else:
        dsx2['field_analog'] = dsx2.field_analog + abs(dsx2.field.mean() - dsx2.field_analog.mean())  
    
    ## Make Vxx positive if measured negative
    if dsx1.Vxx.mean()<0:
        dsx1['Vxx'] = -dsx1['Vxx']   
    if dsx2.Vxx.mean()<0:
        dsx2['Vxx'] = -dsx2['Vxx']
        
    if dsx1.Vxy.mean()<0:
        dsx1['Vxy'] = -dsx1['Vxy']
    if dsx2.Vxy.mean()<0:
        dsx2['Vxy'] = -dsx2['Vxy']
        
    aspect_ratio = 10
    dsx1['rho_xx'] = dsx1.Vxx/dsx1.Iac/aspect_ratio
    dsx2['rho_xx'] = dsx2.Vxx/dsx2.Iac/aspect_ratio
    dsx1['rho_xy'] = dsx1.Vxy/dsx1.Iac
    dsx2['rho_xy'] = dsx2.Vxy/dsx2.Iac
    
    if top_peaks==True:
        peaks1, _ = find_peaks(dsx1.rho_xx, prominence=prominence)
        peaks2, _ = find_peaks(dsx2.rho_xx, prominence=prominence)
        lenpeaks = np.min([len(peaks1), len(peaks2)])
        diflen = np.max([len(peaks1)-lenpeaks, len(peaks2)-lenpeaks])
    else:
        peaks1, _ = find_peaks(-dsx1.rho_xx, prominence=prominence)
        peaks2, _ = find_peaks(-dsx2.rho_xx, prominence=prominence)
        lenpeaks = np.min([len(peaks1), len(peaks2)])
        diflen = np.max([len(peaks1)-lenpeaks, len(peaks2)-lenpeaks])
    
    if len(peaks1)>len(peaks2):
        peaks1 = peaks1[diflen:]
    elif len(peaks2)>len(peaks1):
        peaks2 = peaks2[diflen:]
    
    if field_shift == None:
        field_shift = np.mean(np.array(dsx1.field_analog[peaks1]) - np.array(dsx2.field_analog[peaks2]))
    else:
        field_shift = field_shift
    dsx2['field_analog'] = dsx2.field_analog + field_shift
    
    ## find common field analog data
    max_minfield = np.max([dsx1.field_analog.min(), dsx2.field_analog.min()])
    min_maxfield = np.min([dsx1.field_analog.max(), dsx2.field_analog.max()])
    len_field = np.max([dsx1.field_analog.size, dsx2.field_analog.size])
    field_intp = np.linspace(max_minfield, min_maxfield, len_field)
    
    ## interpolate rho_xx
    rho_xx1_intp = np.interp(field_intp, dsx1.field_analog, dsx1.rho_xx)
    rho_xx2_intp = np.interp(field_intp, dsx2.field_analog, dsx2.rho_xx)
    rho_xx_sym = (rho_xx1_intp + rho_xx2_intp) / 2
    
    ## interpolate rho_xy
    rho_xy1_intp = np.interp(field_intp, dsx1.field_analog, dsx1.rho_xy)
    rho_xy2_intp = np.interp(field_intp, dsx2.field_analog, dsx2.rho_xy)
    rho_xy_sym = (rho_xy1_intp + rho_xy2_intp) / 2
    
    if plot_figures==True:
        ## Plot original data with peaks found to shift data
        fig, ax = plt.subplots(figsize=(14,8))
        ax.plot(dsx1.field_analog, dsx1.rho_xx, label='data set 1')
        ax.plot(dsx2.field_analog, dsx2.rho_xx, label='data set 2')
        ax.plot(dsx1.field_analog[peaks1], dsx1.rho_xx[peaks1], 'rd')
        ax.plot(dsx2.field_analog[peaks2], dsx2.rho_xx[peaks2], 'go')
        ax.set_xlabel('Bfield (T)', fontsize=18)
        ax.set_ylabel('$\\rho_{xx}$ (\u03A9)', fontsize=18)
        ax.plot(field_intp, rho_xx_sym, label='symmetrized data')
        ax.legend()
        fig.tight_layout()    
    
    if plot_figures==True:
        fig, ax = plt.subplots(figsize=(14,8))
        ax.plot(dsx1.field_analog, dsx1.rho_xy, label='data set 1')
        ax.plot(dsx2.field_analog, dsx2.rho_xy, label='data set 2')
        ax.plot(field_intp, rho_xy_sym, label='symmetrized data')
        ax.set_xlabel('Bfield (T)', fontsize=18)
        ax.set_ylabel('$\\rho_{xy}$ (\u03A9)', fontsize=18)
        ax.legend()
        fig.tight_layout()
    return rho_xx_sym, rho_xy_sym, field_intp

#%%
path = r'W:/staff-groups/tnw/ns/qt/ScappucciLab/0_Group members/Lucas/Publications/APL_2023/DataRepository/Fig3/panel_c'

## High density
uuids_high = [1677160811561750695, 1677161733255750695]

## Low density
uuids_low = [1677170062140750695, 1677170984440750695]

## Get symmetrized rho_xy and rho_xx data from two datasets, one for positive magnetic field, one for negative magnetic field.
## If peaks of rho_xx are not identified correctly, change the prominence.
## High density
rho_xx_sym_high, rho_xy_sym_high, field_intp_high = symmetrize(uuids_high, aspect_ratio=10, prominence=15, top_peaks=1, plot_figures=1)
density_high = getDensity(rho_xy_sym_high, field_intp_high, 0.1)
print('dens= '+str(density_high/1e11) +' (1e11/cm2)')

## Low density
rho_xx_sym_low, rho_xy_sym_low, field_intp_low = symmetrize(uuids_low, aspect_ratio=10, prominence=15, top_peaks=1, plot_figures=1)
density_low = getDensity(rho_xy_sym_low, field_intp_low, 0.1)
print('dens= '+str(density_low/1e11) +' (1e11/cm2)')

#%%
inch = 0.39370078740157477

## Define figure presets
mpl.rcdefaults()
mpl.rcParams['figure.dpi'] = 200 
mpl.rcParams['figure.figsize'] = (3.5*inch,2.5*inch) 

mpl.rcParams['axes.linewidth'] = 0.5
mpl.rcParams['lines.linewidth'] = 1
mpl.rcParams["legend.numpoints"] = 1
mpl.rcParams["legend.frameon"] = False
#mpl.rcParams['figure.figsize'] = (4, 3)
mpl.rcParams['font.size'] = 10
mpl.rcParams['font.family'] = 'Arial'

# #Setting finally the Text characters to regular instead of italics.
mpl.rcParams['mathtext.rm'] = 'Arial'                 #'custom''Arial' #custom make mu italic, Arial straight
mpl.rcParams['mathtext.it'] = 'Arial: italic'         #make latex italics unless use \mathrm{}
mpl.rcParams['mathtext.default'] = 'it' #'default'
mpl.rcParams['xtick.direction'] = 'in'
mpl.rcParams['ytick.direction'] = 'in'
mpl.rcParams['xtick.major.width'] = 0.5
mpl.rcParams['ytick.major.width'] = 0.5
mpl.rcParams['xtick.major.size'] = 3.5 #default 3.5
mpl.rcParams['xtick.minor.size'] = 2 #default 2.0
mpl.rcParams['ytick.major.size'] = 3.5 #default 3.5
mpl.rcParams['ytick.minor.size'] = 2 #default 2.0

mpl.rcParams["errorbar.capsize"]= 3
mpl.rcParams['savefig.bbox'] = 'tight' # to avoid clipping mask cropping the axis label

## Setup figure
two_col_width = 8.5 # width of figure in cm 
sep = .04
cbar_fontsize = 6
factor = 185 / 155.514

fig = plt.figure(figsize=(two_col_width*factor*inch, 13*inch), constrained_layout=0)
gs1 = gridspec.GridSpec(2, 1, figure=fig, width_ratios=[1], height_ratios=[1,1],
               wspace=0, hspace=0.2, left=None) # hspace=0.05*5

ax1 = fig.add_subplot(gs1[0])
ax2 = fig.add_subplot(gs1[1])


def plt_rho_xx_field_high(fig, ax, rho_xx_high, field_high):    
    rho_xx = rho_xx_high
    field = field_high
    
    ax.plot(field, rho_xx, 'k', lw=1)
    ax.set_ylim([5, 15])
    ax.set_ylabel('$\\rho_{\mathdefault{xx}}$ (\u03A9)', horizontalalignment='right', labelpad=5)
    
    ax.set_xlim([0.04, 0.310])
    ax.set_xticklabels([])
    ax.text(0.12,10, '$B_\mathdefault{c}^\mathdefault{odd}$', fontsize=8)
    ax.text(0.08,10, '$B_\mathdefault{c}^\mathdefault{even}$', fontsize=8)

def plt_rho_xx_field_low(fig, ax, rho_xx_low, field_low):
    rho_xx = rho_xx_low
    field = field_low
    
    ax.plot(field, rho_xx, 'k', lw=1)
    ax.set_ylim([25, 120])
    # ax.set_ylabel('$\\rho_{\mathrm{xx}}$ (\u03A9)')
    ax.text(0.22,105, '$B_\mathdefault{c}^\mathdefault{even}$', fontsize=8)
    ax.text(0.09,70, '$B_\mathdefault{c}^\mathdefault{odd}$', fontsize=8)    
    ax.set_xlim([0.04, 0.310])
    ax.set_xlabel('$B$ (T)')

plt_rho_xx_field_high(fig, ax1, rho_xx_sym_high, field_intp_high)
plt_rho_xx_field_low(fig, ax2, rho_xx_sym_low, field_intp_low)





